/*
 * \file handler.c
 *
 * \date Feb 15, 2017
 * \author Ting
 */

#include "handler.h"
#include "ledRGB.h"
#include "lcdled.h"

/*! 
 * vari&aacute;vel de controle da contagem de tempo (m&uacute;ltiplo de 0.25s)
 */
extern uint32_t tempo;    

/*!
 * vari&aacute;vel de cor selecionada
 */         
extern uint8_t indice;            

/*!
 * vari&aacute;vel de estado da botoeira PTA12 (1: pressionado; 0 solto)
 */
extern uint8_t estado_pta12;     

/*!
 * varivel de estado do ledRGB
 */
extern uint8_t apagado;

/*!
 * vetor de cores do led
 */
extern cor cores_estaticas[];

/*!
 * \fn PORTA_IRQHandler (void)
 * \brief Rotina de servio para GPIOA (Vetor 46, IRQ30, prioridade em NVICIPR7[31:30])
 */
void PORTA_IRQHandler(void) {
	static uint32_t tempo1=0, tempo2=0;
	float m;
	
	if (PORTA_PCR5 & PORT_PCR_ISF_MASK) {
		PORTA_PCR5 |= PORT_PCR_ISF_MASK;            ///< abaixa a bandeira de interrup&ccedil;&atilde;o
	} else if (PORTA_PCR12 & PORT_PCR_ISF_MASK) {   
		if (GPIOA_PDIR & GPIO_PIN(12)) {
			/*! Soltou */
			estado_pta12 = 0;
			tempo2 = SYST_CVR;
	        if (tempo2 > tempo1) m = (tempo2-tempo1)/((float)(SYST_RVR));
	        else m = (SYST_RVR-tempo1+tempo2)/((float)(SYST_RVR));
			indice = (tempo > 0)?(tempo+(SYST_RVR-tempo1+tempo2)/((float)SYST_RVR)):(int)m;   ///< computa tempo transcorrido em mltiplo de 0.25s
			// SYST_CSR &= ~SysTick_CSR_ENABLE_MASK;    ///< desabilita o clock do SysTick
            enviaLed(0x00);               ///< apaga leds vermelhos
		} else {
			/*! Apertou */
            enviaLed(0xFF);               ///< liga leds vermelhos 
			estado_pta12 = 1;
			tempo1 = SYST_CVR;
			tempo = 0;         ///< inicia a contagem de tempo
			// SYST_CSR |= SysTick_CSR_ENABLE_MASK;     ///< habilita o clock do SysTick       
		}
		PORTA_PCR12 |= PORT_PCR_ISF_MASK;            ///< abaixa a bandeira de interrup&ccedil;&atilde;o
	} 
}

/*!
 * \fn SysTick_Handler (void)
 * \brief Rotina de servio para exce&ccedil&atilde;o SysTick (Vetor 15)
 */
void SysTick_Handler(void) {
	tempo++;	
}

/**
 * \fn PIT_IRQHandler (void)
 * 
 * \brief Rotina de servio para PIT (Vetor 38, IRQ22, prioridade em NVICIPR5[23:22])
 * 
 */
void PIT_IRQHandler(void) {
	//GPIOD_PTOR |= GPIO_PTOR_PTTO(GPIO_PIN(1));  ///< GPIOD_PTOR[1]=1 (alterna o valor)
    /*!
     * Exibe a cor piscante conforme o intervalo de tempo estimado
     */
	if (apagado) {
		apagaLedRGB();
		apagado = 0;
	} else {
		led (VERMELHO, *((estado_led *)(cores_estaticas[indice].vermelho)));
		led (VERDE, *((estado_led *)(cores_estaticas[indice].verde)));
		led (AZUL, *((estado_led *)(cores_estaticas[indice].azul)));
		apagado = 1;
	}

	PIT_TFLG0 |= PIT_TFLG_TIF_MASK;     ///< TIF=PIT_TFLG0[31]: w1c (limpa pendncias)
}

/**
 * \brief non-maskable interrupt handler
 * \note A fun&ccedil;&atilde;o do PTA4 &eacute; NMI. Se habilitar no NVIC o processamento
 * da interrup&ccedil&atilde;o da porta A, pode-se gerar interrup&ccedil;&otilde;es inesperadas.
 * Uma solu&ccedil;&atilde;o &eacute; criar uma rotina de tratamente vazia 
 * (entra e sai sem fazer nada). 
 */
void NMI_Handler (void) {
}
