/*!
 * \file main.c
 * \brief Captura dos eventos de push-button por mecanismo de interrup&ccedil;&atilde;o.
 * \details Este c&oacute;digo ilustra o mecanismo de interrup&ccedil;&atilde;o dispon&iacute;vel no MKZ25Z128 
 *          para captura de um evento externo ass &iacute;ncrono;.
 * \mainpage Projeto de controle da cor dos leds RGB por intervalos de tempo.
 * \author Wu Shin-Ting
 * \date 24/02/2017
 * 
 * \attention 
 * 
 * \note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
 */

/*!
 * \fn main(void)
 * \attention Diferentes formas de declara&ccedil;&atilde;o e atribui&ccedil;&atilde;o 
 * s&atilde;o utilizadas. O importante &eacute; alocar um espa&ccedil;o de mem&oacute;ria
 * suficiente para armazenar a quantidade de caracteres (um byte por caractere).  
 *       
 */

#include "derivative.h" 
#include "stdlib.h"
#include "string.h"

#include "nvic.h"
#include "atrasos.h"
#include "pushbutton.h"
#include "ledRGB.h"
#include "lcdled.h"
#include "systick.h"
#include "pit.h"
/*! 
 * vari&aacute;vel de controle da contagem de tempo (m&uacute;ltiplo de 0.25s)
 */
uint32_t tempo;       

/*!
 * vari&aacute;vel de cor selecionada
 */         
uint8_t indice;            

/*!
 * vari&aacute;vel de estado da botoeira PTA12 (1: pressionado; 0 solto)
 */
uint8_t estado_pta12;       

/*!
 * varivel de estado do ledRGB
 */
uint8_t apagado;

/*!
 * vetor de cores do led
 */
cor cores_estaticas[] = {{ACESO,APAGADO,APAGADO},{ACESO,ACESO,ACESO},{ACESO,ACESO,APAGADO},
				{APAGADO,ACESO,ACESO},{ACESO,APAGADO,ACESO},{APAGADO,APAGADO,APAGADO}};

int main(void)
{
	/*!
	 * Declara&ccedil;&atilde;o e inicializa&ccedil;&atilde;o de strings de cores
	 */
	char *str0 = "VERMELHO";
	char *str1 = "BRANCO";
	char *str2 = "AMARELO";
	char *str3 = "CIANO"; 
	char *str4 = "MAGENTA";
	char *str5 = "PRETO";
	
	char *str;
	char str6[2];
	uint8_t antes;
	
	/*!
	 * Inicializa&ccedil;&atilde;o do ledRGB
	 */
	initLedRGB();
	
    /*!
     * Inicializa&ccedil;&otilde;es dos push buttons com interrup&ccedil;&atilde;o habilitada
     */
    initPushbuttonIRQA();
    enablePushbuttonIRQA();
    
    /*!
     * Inicializa&ccedil;&otilde;es do LCD
     */
    initLcdledGPIO();
    initLCD();
    
    
    /*!
     * Inicializa&ccedil;&atilde;o do temporizador PIT com contagem 0xA00000
     */
    initPIT(0xA00000);
    
	/*! 
	 * Habilita a interrup&ccedil;&otilde;es no controlador NVIC  
	 */
    enableNVICPTA(3);
    enableNVICPIT(1);
	
	/*! Inicialize o Timer do Processador */
	initSysTick();

	estado_pta12 = 0; ///< estado inicial (pta12 solto)
	antes = 0;        ///< estado anterior diferente do estado corrente
	indice = 5;       ///< ledRGB PRETO = apagado
	apagado = 1;       
	
	for(;;) {
		/*!
		 * Novo processamento somente quando h&aacute; mudan&ccedil;a do estado da cor
		 */
        if (indice != antes)
        {
        	/*! 
        	 * Selecione a mensagem apropriada e habilita a interrup&ccedil;&atilde;o do PIT 
        	 * quando a cor n&atilde;o for preta
        	 */
        	switch (indice) {
        	case 0: 
        		str = str0;
        		enablePITinterrup();
        		break;
        	case 1: 
        		str = &str1[0];
        		enablePITinterrup();
        		break;
        	case 2:
        		str = &str2[0];
        		enablePITinterrup();
        		break;
        	case 3: 
        		str = str3;
        		enablePITinterrup();
        		break;
        	case 4:
        		str = str4;
        		enablePITinterrup();
        		break;
        	case 5:
        		str = str5;
        		apagaLedRGB();
        		disablePITinterrup();
        		break;
        	}
        	antes = indice;   ///< atualiza a variavel antes
        	/*! Envie mensagem para LCD e leds */
        	ConvUI2String (indice, str6);
        	limpaLCD();
        	mandaString(str6);
        	mandaString("  ");
        	mandaString(str);
        } else if (indice > 5){
        	/*! passou o intervalo maximo permitido */
        	indice = 5;
        }
	}
}
